

<!-- @import "[TOC]" {cmd="toc" depthFrom=2 depthTo=5 orderedList=false} -->

<!-- code_chunk_output -->

- [Share](#share)
  - [Overview](#overview)
  - [ShareService](#shareservice)

<!-- /code_chunk_output -->



# Share

## Overview

## ShareService

ShareService is a helper library that gives some feature utility for sharing. It has `showBottomSheet` method that displays a bottom sheet showing some UI elements for sharing. You may get an idea seeing the look and the code of the method.

To use share service, initialize like below;
 
```dart
ShareService.instance.init(uriPrefix: uriPrefix, appId: appId);
```

`uriPrefix` is one of the `URL prefixes` from the Firebase Dynamic Link page. It would look like `https://xxxx.page.link`. And the `appId` is the package name for Android and bundle id for iOS. If you have different `appId` for different platform, you may use tertiary operator like below.


Example

```dart
ShareService.instance.init(
  uriPrefix: "https://myapp.page.link",
  appId: isAndroid ? "com.myapp.android" : "com.myapp.other",
);
```


It also has a method `dyamicLink` that returns a short dyanmic link. You can call this method to get the dynamic link and share.

The exmaple below is using `SharePlus` package to share the link generated by `ShareService.instnace.dyanmicink`.

```dart
Share.share(
  await ShareService.instance.dynamicLink(
    type: 'post',
    id: '-post-id-xxx-',
    title: 'Post title',
    description: 'Post content ...',
  ),
  subject: post.title,
);
```

See the source code of the method to know better about the parameter options.

For now, it supports `user` and `post` type of content. When the type is `user`, it will generate a link to open the user's public profile. And of course, when the link is opened, the app should show the user's public profile. This goes same to post.

We don't support the link for `chat` yet. Because the user needs to register first before entering the chat room while user profile and post view can be seen without login.


Note that, using `SharePlus` to share the dynamic link is enough. But fireflutter provides some UI widgets to make it look different. You can try them or you can simply stick to `SharePlus`.

Dispaly a share bottom sheet.

```dart
ShareService.instance.showBottomSheet(actions: [
  IconTextButton(
    icon: const Icon(Icons.share),
    label: "Share",
    onTap: () async {
      Share.share(
        await ShareService.instance.dynamicLink(
          link: "https://xxxx.page.link/?type=xxx&id=xxx",
          uriPrefix: "https://xxxx.page.link",
          appId: "xxx",
          title: 'title',
          description: 'description..',
        ),
        subject: 'subject',
      );
    },
  ),
  IconTextButton(
    icon: const Icon(Icons.copy),
    label: "Copy Link",
    onTap: () {},
  ),
  IconTextButton(
    icon: const Icon(Icons.comment),
    label: "Message",
    onTap: () {},
  ),
]),
```

Example of copying the dynamic link to clipboard

```dart
IconTextButton(
  icon: const Icon(Icons.copy),
  label: "Copy Link",
  onTap: () async {
    final link = await ShareService.instance.dynamicLink(
      type: DynamicLink.post.name,
      id: 'post.id',
      title: 'title',
      description: 'description..',
    );
    Clipboard.setData(ClipboardData(text: link));
    toast(title: tr.copyLink, message: tr.copyLinkMessage);
  },
),
```

Below is an example of how to use `ShareBottomSheet` widget. You can insert this widget in home screen and do some UI work. Then, apply it.

```dart
ShareBottomSheet(actions: [
  IconTextButton(
    icon: const Icon(Icons.share),
    label: "Share",
    onTap: () async {
      Share.share(
        await ShareService.instance.dynamicLink(
          type: DynamicLink.feed.name,
          id: 'post.id',
          title: 'title',
          description: 'description..',
        ),
        subject: 'subject',
      );
    },
  ),
  IconTextButton(
    icon: const Icon(Icons.copy),
    label: "Copy Link",
    onTap: () async {
      final link = await ShareService.instance.dynamicLink(
        type: DynamicLink.post.name,
        id: 'post.id',
        title: 'title',
        description: 'description..',
      );
      Clipboard.setData(ClipboardData(text: link));
      toast(title: tr.copyLink, message: tr.copyLinkMessage);
    },
  ),
  IconTextButton(
    icon: const Icon(Icons.comment),
    label: "Message",
    onTap: () async {
      final link = await ShareService.instance.dynamicLink(
        type: DynamicLink.post.name,
        id: 'post.id',
        title: 'title',
        description: 'description..',
      );
      final re = await launchSMS(phnumber: '', msg: link);
      if (re) {
        toast(title: 'SMS', message: 'Link sent by SMS');
      } else {
        toast(title: 'SMS', message: 'Cannot open SMS');
      }
    },
  ),
]),
```

